/* -*-c-*-
 *
 * $Id: ghash.h,v 1.7 2004/04/08 01:36:15 mdw Exp $
 *
 * Generic hash function interface
 *
 * (c) 1999 Straylight/Edgeware
 */

/*----- Licensing notice --------------------------------------------------* 
 *
 * This file is part of Catacomb.
 *
 * Catacomb is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Library General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 * 
 * Catacomb is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Library General Public License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with Catacomb; if not, write to the Free
 * Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
 * MA 02111-1307, USA.
 */

#ifndef CATACOMB_GHASH_H
#define CATACOMB_GHASH_H

#ifdef __cplusplus
  extern "C" {
#endif

/*----- Header files ------------------------------------------------------*/

#include <stddef.h>

#include <mLib/bits.h>

/*----- Generic hash function interface -----------------------------------*/

typedef struct ghash {
  const struct ghash_ops *ops;		/* Pointer to hash operations */
} ghash;

typedef struct ghash_ops {
  const struct gchash *c;		/* Pointer to hash class */
  void (*hash)(ghash */*h*/, const void */*p*/, size_t /*sz*/); /* Hash */
  octet *(*done)(ghash */*h*/, void */*buf*/); /* Write result */
  void (*destroy)(ghash */*h*/);	/* Destroy hash block */
  ghash *(*copy)(ghash */*h*/);		/* Make a copy of the hash context */
} ghash_ops;

#define GH_INIT(ch)		(ch)->init()
#define GH_CLASS(h)		(h)->ops->c
#define GH_HASH(h, p, sz)	(h)->ops->hash((h), (p), (sz))
#define GH_DONE(h, buf)		(h)->ops->done((h), (buf))
#define GH_DESTROY(h)		(h)->ops->destroy((h))
#define GH_COPY(h)		(h)->ops->copy((h))

typedef struct gchash {
  const char *name;			/* Name of the hash function */
  size_t hashsz;			/* Size of output hash */
  ghash *(*init)(void);			/* Create a new hash instance */
  size_t bufsz;				/* Buffer size, or zero */
} gchash;

/*----- Tables ------------------------------------------------------------*/

extern const gchash *const ghashtab[];

/* --- @ghash_byname@ --- *
 *
 * Arguments:	@const char *p@ = pointer to name string
 *
 * Returns:	The named cipher class, or null.
 */

extern const gchash *ghash_byname(const char */*p*/);

/*----- That's all, folks -------------------------------------------------*/

#ifdef __cplusplus
  }
#endif

#endif
